@@ -1,20 +1,14 @@
-"""
-Commonly used dialogs
-isort:skip_file
-"""
+"""Commonly used dialogs"""
 import os
 from gettext import gettext as _
 
-import gi
-gi.require_version("WebKit2", "4.0")
-from gi.repository import GObject, Gtk, WebKit2, GLib
+from gi.repository import GLib, GObject, Gtk
 
 from lutris import api, settings
-from lutris.database import sources as sources_db
 from lutris.gui.widgets.log_text_view import LogTextView
 from lutris.util import datapath
-from lutris.util.log import logger
 from lutris.util.jobs import AsyncCall
+from lutris.util.log import logger
 
 
 class Dialog(Gtk.Dialog):
@@ -93,14 +87,15 @@ class NoticeDialog(Gtk.MessageDialog):
 
 
 class ErrorDialog(Gtk.MessageDialog):
-
     """Display an error message."""
 
     def __init__(self, message, secondary=None, parent=None):
         super().__init__(buttons=Gtk.ButtonsType.OK, parent=parent)
-        self.set_markup(message)
+        # Gtk doesn't wrap long labels containing no space correctly
+        # the length of the message is limited to avoid display issues
+        self.set_markup(message[:256])
         if secondary:
-            self.format_secondary_text(secondary)
+            self.format_secondary_text(secondary[:256])
         self.run()
         self.destroy()
 
@@ -116,6 +111,8 @@ class QuestionDialog(Gtk.MessageDialog):
         super().__init__(message_type=Gtk.MessageType.QUESTION, buttons=Gtk.ButtonsType.YES_NO)
         self.set_markup(dialog_settings["question"])
         self.set_title(dialog_settings["title"])
+        if "parent" in dialog_settings:
+            self.set_transient_for(dialog_settings["parent"])
         if "widgets" in dialog_settings:
             for widget in dialog_settings["widgets"]:
                 self.get_message_area().add(widget)
@@ -123,25 +120,28 @@ class QuestionDialog(Gtk.MessageDialog):
         self.destroy()
 
 
-class DirectoryDialog(Gtk.FileChooserDialog):
+class DirectoryDialog:
 
     """Ask the user to select a directory."""
 
     def __init__(self, message, default_path=None, parent=None):
-        super().__init__(
-            title=message,
-            action=Gtk.FileChooserAction.SELECT_FOLDER,
-            buttons=(_("_Cancel"), Gtk.ResponseType.CLOSE, _("_OK"), Gtk.ResponseType.OK),
-            parent=parent,
+        self.folder = None
+        dialog = Gtk.FileChooserNative.new(
+            message,
+            parent,
+            Gtk.FileChooserAction.SELECT_FOLDER,
+            _("_OK"),
+            _("_Cancel"),
         )
         if default_path:
-            self.set_current_folder(default_path)
-        self.result = self.run()
-        self.folder = self.get_current_folder()
-        self.destroy()
+            dialog.set_current_folder(default_path)
+        self.result = dialog.run()
+        if self.result == Gtk.ResponseType.ACCEPT:
+            self.folder = dialog.get_filename()
+        dialog.destroy()
 
 
-class FileDialog(Gtk.FileChooserDialog):
+class FileDialog:
 
     """Ask the user to select a file."""
 
@@ -153,20 +153,21 @@ class FileDialog(Gtk.FileChooserDialog):
             action = Gtk.FileChooserAction.SAVE
         else:
             action = Gtk.FileChooserAction.OPEN
-        super().__init__(
+        dialog = Gtk.FileChooserNative.new(
             message,
             None,
             action,
-            (_("_Cancel"), Gtk.ResponseType.CANCEL, _("_OK"), Gtk.ResponseType.OK),
+            _("_OK"),
+            _("_Cancel"),
         )
         if default_path and os.path.exists(default_path):
-            self.set_current_folder(default_path)
-        self.set_local_only(False)
-        response = self.run()
-        if response == Gtk.ResponseType.OK:
-            self.filename = self.get_filename()
+            dialog.set_current_folder(default_path)
+        dialog.set_local_only(False)
+        response = dialog.run()
+        if response == Gtk.ResponseType.ACCEPT:
+            self.filename = dialog.get_filename()
 
-        self.destroy()
+        dialog.destroy()
 
 
 class LutrisInitDialog(Gtk.Dialog):
@@ -177,14 +178,15 @@ class LutrisInitDialog(Gtk.Dialog):
         self.set_border_width(24)
         self.set_decorated(False)
         vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 12)
-        label = Gtk.Label(_("Lutris is starting..."))
+        label = Gtk.Label(_("Checking for runtime updates, please wait…"))
         vbox.add(label)
         self.progress = Gtk.ProgressBar(visible=True)
         self.progress.set_pulse_step(0.1)
         vbox.add(self.progress)
         self.get_content_area().add(vbox)
-        GLib.timeout_add(125, self.show_progress)
+        self.progress_timeout = GLib.timeout_add(125, self.show_progress)
         self.show_all()
+        self.connect("destroy", self.on_destroy)
         AsyncCall(self.initialize, self.init_cb, init_lutris)
 
     def show_progress(self):
@@ -199,13 +201,16 @@ class LutrisInitDialog(Gtk.Dialog):
             ErrorDialog(str(error))
         self.destroy()
 
+    def on_destroy(self, window):
+        GLib.source_remove(self.progress_timeout)
+        return True
+
 
 class InstallOrPlayDialog(Gtk.Dialog):
 
     def __init__(self, game_name):
         Gtk.Dialog.__init__(self, _("%s is already installed") % game_name)
         self.connect("delete-event", lambda *x: self.destroy())
-
         self.action = "play"
         self.action_confirmed = False
 
@@ -213,7 +218,6 @@ class InstallOrPlayDialog(Gtk.Dialog):
         self.set_border_width(12)
         vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 6)
         self.get_content_area().add(vbox)
-
         play_button = Gtk.RadioButton.new_with_label_from_widget(None, _("Launch game"))
         play_button.connect("toggled", self.on_button_toggled, "play")
         vbox.pack_start(play_button, False, False, 0)
@@ -239,66 +243,50 @@ class InstallOrPlayDialog(Gtk.Dialog):
         self.destroy()
 
 
-class PgaSourceDialog(GtkBuilderDialog):
-    glade_file = "dialog-pga-sources.ui"
-    dialog_object = "pga_dialog"
+class LaunchConfigSelectDialog(Gtk.Dialog):
+    def __init__(self, game, configs):
+        Gtk.Dialog.__init__(self, _("Select game to launch"))
+        self.connect("delete-event", lambda *x: self.destroy())
+        self.config_index = 0
+        self.confirmed = False
+        self.set_size_request(320, 120)
+        self.set_border_width(12)
+        vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 6)
+        self.get_content_area().add(vbox)
 
-    def __init__(self, parent=None):
-        super(PgaSourceDialog, self).__init__(parent=parent)
+        primary_game_radio = Gtk.RadioButton.new_with_label_from_widget(None, game.name)
+        primary_game_radio.connect("toggled", self.on_button_toggled, 0)
+        vbox.pack_start(primary_game_radio, False, False, 0)
+        for i, config in enumerate(configs):
+            _button = Gtk.RadioButton.new_from_widget(primary_game_radio)
+            _button.set_label(config["name"])
+            _button.connect("toggled", self.on_button_toggled, i + 1)
+            vbox.pack_start(_button, False, False, 0)
 
-        # GtkBuilder Objects
-        self.sources_selection = self.builder.get_object("sources_selection")
-        self.sources_treeview = self.builder.get_object("sources_treeview")
-        self.remove_source_button = self.builder.get_object("remove_source_button")
+        button_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 6)
+        button_box.set_halign(Gtk.Align.END)
+        cancel_button = Gtk.Button(_("Cancel"))
+        cancel_button.connect("clicked", self.on_cancel)
+        button_box.pack_start(cancel_button, False, False, 0)
 
-        # Treeview setup
-        self.sources_liststore = Gtk.ListStore(str)
-        renderer = Gtk.CellRendererText()
-        renderer.set_padding(4, 10)
-        uri_column = Gtk.TreeViewColumn("URI", renderer, text=0)
-        self.sources_treeview.append_column(uri_column)
-        self.sources_treeview.set_model(self.sources_liststore)
-        sources = sources_db.read_sources()
-        for __, source in enumerate(sources):
-            self.sources_liststore.append((source, ))
+        confirm_button = Gtk.Button(_("OK"))
+        confirm_button.connect("clicked", self.on_confirm)
+        button_box.pack_start(confirm_button, False, False, 0)
+        vbox.pack_start(button_box, False, False, 0)
 
-        self.remove_source_button.set_sensitive(False)
-        self.dialog.show_all()
+        self.show_all()
+        self.run()
 
-    @property
-    def sources_list(self):
-        return [source[0] for source in self.sources_liststore]
+    def on_button_toggled(self, _button, index):
+        self.config_index = index
 
-    def on_apply(self, widget, data=None):
-        sources_db.write_sources(self.sources_list)
-        self.on_close(widget, data)
+    def on_cancel(self, _button):
+        self.confirmed = False
+        self.destroy()
 
-    def on_add_source_button_clicked(self, widget, data=None):  # pylint: disable=unused-argument
-        chooser = Gtk.FileChooserDialog(
-            _("Select directory"),
-            self.dialog,
-            Gtk.FileChooserAction.SELECT_FOLDER,
-            (_("_Cancel"), Gtk.ResponseType.CANCEL, _("_OK"), Gtk.ResponseType.OK),
-        )
-        chooser.set_local_only(False)
-        response = chooser.run()
-        if response == Gtk.ResponseType.OK:
-            uri = chooser.get_uri()
-            if uri not in self.sources_list:
-                self.sources_liststore.append((uri, ))
-        chooser.destroy()
-
-    def on_remove_source_button_clicked(self, widget, data=None):  # pylint: disable=unused-argument
-        """Remove a source."""
-        (model, treeiter) = self.sources_selection.get_selected()
-        if treeiter:
-            # TODO : Add confirmation
-            model.remove(treeiter)
-
-    def on_sources_selection_changed(self, widget, data=None):  # pylint: disable=unused-argument
-        """Set sensitivity of remove source button."""
-        _, treeiter = self.sources_selection.get_selected()
-        self.remove_source_button.set_sensitive(treeiter is not None)
+    def on_confirm(self, _button):
+        self.confirmed = True
+        self.destroy()
 
 
 class ClientLoginDialog(GtkBuilderDialog):
@@ -348,103 +336,6 @@ class ClientLoginDialog(GtkBuilderDialog):
             self.dialog.destroy()
 
 
-class WebConnectDialog(Dialog):
-    """Login form for external services"""
-
-    def __init__(self, service, parent=None):
-
-        self.context = WebKit2.WebContext.new()
-        if "http_proxy" in os.environ:
-            proxy = WebKit2.NetworkProxySettings.new(os.environ["http_proxy"])
-            self.context.set_network_proxy_settings(WebKit2.NetworkProxyMode.CUSTOM, proxy)
-        WebKit2.CookieManager.set_persistent_storage(
-            self.context.get_cookie_manager(),
-            service.cookies_path,
-            WebKit2.CookiePersistentStorage(0),
-        )
-        self.service = service
-
-        super(WebConnectDialog, self).__init__(title=service.name, parent=parent)
-        self.set_border_width(0)
-        self.set_default_size(390, 500)
-
-        self.webview = WebKit2.WebView.new_with_context(self.context)
-        self.webview.load_uri(service.login_url)
-        self.webview.connect("load-changed", self.on_navigation)
-        self.webview.connect("create", self.on_webview_popup)
-        self.vbox.pack_start(self.webview, True, True, 0)  # pylint: disable=no-member
-
-        webkit_settings = self.webview.get_settings()
-        # Allow popups (Doesn't work...)
-        webkit_settings.set_enable_write_console_messages_to_stdout(True)
-        webkit_settings.set_allow_modal_dialogs(True)
-        # Enable developer options for troubleshooting (Can be disabled in
-        # releases)
-        webkit_settings.set_javascript_can_open_windows_automatically(True)
-        webkit_settings.set_enable_developer_extras(True)
-
-        self.show_all()
-
-    def enable_inspector(self):
-        """If you want a full blown Webkit inspector, call this"""
-        inspector = self.webview.get_inspector()
-        inspector.show()
-
-    def on_navigation(self, widget, load_event):
-        if load_event == WebKit2.LoadEvent.FINISHED:
-            url = widget.get_uri()
-            if url.startswith(self.service.redirect_uri):
-                self.service.request_token(url)
-                self.destroy()
-        return True
-
-    def on_webview_popup(self, widget, navigation_action):
-        """Handles web popups created by this dialog's webview"""
-        uri = navigation_action.get_request().get_uri()
-        view = WebKit2.WebView.new_with_related_view(widget)
-        view.load_uri(uri)
-        popup_dialog = WebPopupDialog(view, parent=self)
-        popup_dialog.set_modal(True)
-        popup_dialog.show()
-        return view
-
-
-class WebPopupDialog(Dialog):
-    """Dialog for handling web popups"""
-
-    def __init__(self, webview, parent=None):
-        # pylint: disable=no-member
-        self.parent = parent
-        super(WebPopupDialog, self).__init__(title=_('Loading...'), parent=parent)
-        self.webview = webview
-        self.webview.connect("ready-to-show", self.on_ready_webview)
-        self.webview.connect("notify::title", self.on_available_webview_title)
-        self.webview.connect("create", self.on_new_webview_popup)
-        self.webview.connect("close", self.on_webview_close)
-        self.vbox.pack_start(self.webview, True, True, 0)
-        self.set_border_width(0)
-        self.set_default_size(390, 500)
-
-    def on_ready_webview(self, webview):
-        self.show_all()
-
-    def on_available_webview_title(self, webview, gparamstring):
-        self.set_title(webview.get_title())
-
-    def on_new_webview_popup(self, webview, navigation_action):
-        """Handles web popups created by this dialog's webview"""
-        uri = navigation_action.get_request().get_uri()
-        view = WebKit2.WebView.new_with_related_view(webview)
-        view.load_uri(uri)
-        dialog = WebPopupDialog(view, parent=self)
-        dialog.set_modal(True)
-        dialog.show()
-        return view
-
-    def on_webview_close(self, webview):
-        self.destroy()
-
-
 class InstallerSourceDialog(Gtk.Dialog):
 
     """Show install script source"""
@@ -533,3 +424,45 @@ class WineNotInstalledWarning(DontShowAgainDialog):
             ),
             parent=parent,
         )
+
+
+class MoveDialog(Gtk.Dialog):
+    __gsignals__ = {
+        "game-moved": (GObject.SIGNAL_RUN_FIRST, None, ()),
+    }
+
+    def __init__(self, game, destination):
+        super().__init__()
+
+        self.game = game
+        self.destination = destination
+        self.new_directory = None
+
+        self.set_size_request(320, 60)
+        self.set_border_width(24)
+        self.set_decorated(False)
+        vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 12)
+        label = Gtk.Label(_("Moving %s to %s..." % (game, destination)))
+        vbox.add(label)
+        self.progress = Gtk.ProgressBar(visible=True)
+        self.progress.set_pulse_step(0.1)
+        vbox.add(self.progress)
+        self.get_content_area().add(vbox)
+        GLib.timeout_add(125, self.show_progress)
+        self.show_all()
+
+    def move(self):
+        AsyncCall(self._move_game, self.on_game_moved)
+
+    def show_progress(self):
+        self.progress.pulse()
+        return True
+
+    def _move_game(self):
+        self.new_directory = self.game.move(self.destination)
+
+    def on_game_moved(self, _result, error):
+        if error:
+            ErrorDialog(str(error))
+        self.emit("game-moved")
+        self.destroy()
